"use strict";
var opt = require('node-getopt').create([
    ['m', 'operator=OP', 'Morphological operator (default: none).'],
    ['' , 'project', 'Project frame representation back to basis (default: no).'],
    ['o', 'output=OUT', 'Output file (only valid with a single output).'],
    ['s', 'stem=STEM', 'Stem of outputs (default: input file without extension).'],
    ['h' , 'help'                , 'display this help'],
]).bindHelp(
    "Usage: node filter.js [OPTIONS] INPUT_IMAGE1.png\n" +
    "  Rotation invariant morphological filtering of structure tensors.\n" +
    "\n" +
    "[[OPTIONS]]\n" +
    "\n" +
    "The following operators are available out-of-the-box:\n" +
    "  Dilation, Erosion, Closing, Opening\n" +
    "For the -m/--operator option, append a dash and the structuring element size. " +
    "Note that a structuring element of size 2 will cover three pixels (the 2 refers to the distance between pixel centers).\n"
).parseSystem();
var fs = require('fs');
var png = require('pngjs').PNG;
var frameMorphology = require('frameTensorMorphology');

// Validate and process options
var inputRegExp = /^(.*)\.png$/i;

if (opt.argv.length!=1) { // Check this before determining the stem (at least), as it uses argv.
    console.error("One, and only one, input file is required.\n");
    process.exit(1);
}

if (!inputRegExp.test(opt.argv[0])) {
    console.error("Input filename must end in .png (and the file must in fact be a PNG file).\n");
    process.exit(2);
}

var operator = opt.options.operator ? opt.options.operator : '';
var project = opt.options.project;
var output = opt.options.output;
var stem = opt.options.stem ? opt.options.stem : inputRegExp.exec(opt.argv[0])[1];
var verbose = opt.options.verbose;

// Actual filtering
var op, fr;

function filterCallback() {
    var outputName = output ? output : operator.length>0 ? ([stem, operator].join('-') + (project ? '-projected' : '') + '.png') : (stem + '-gradientMagnitude.png');
    frameMorphology.filter(this, operator, project);
    this.pack().pipe(fs.createWriteStream(outputName));
}

fs.createReadStream(opt.argv[0])
    .pipe(new png())
    .on('parsed', filterCallback);

